==> mls-0.3.5.tar.gz: mls-0.3.5/src/panel_read.cpp <==
///	\file	panel_read.cpp
///	\brief	panel_read.cpp
#include <dirent.h>
#include <sys/stat.h>
#include <time.h>
#include <string>
#include <map>
#include <algorithm>
#include <unistd.h>

#include "panel.h"
#include "conf.h"
#include "strutil.h"
#include "dialog.h"
#include "ZipReader.h"

using namespace std;
using namespace strutil;
using namespace MLS;

// read interface
/*
 * (기본) 파일 리더.
 * file:// 파일 리더
 * arc:// 
 * ftp:// 
 */
 
/// \brief	초기화 함수
void Panel::Init(void)
{
	cur      = 0;
	page     = 0;
	sel_size = 0;
	sel_num  = 0;	

	nfile    = 0;
	ndir     = 0;
	dir_size = 0;	
	
	file_list.clear();
	pool.Clear();	
}

/// \brief	파일명에 해당 되는 색상을 구함
/// \param	t		파일
/// \return	색상
ColorEntry *Panel::GetColor(File &t)
{
	// 이름
	map<string, ColorEntry>::iterator namei = e_tColor.mapName.find( tolower(t.name) );
	
	if (namei != e_tColor.mapName.end() ) 
		return &(*namei).second;
	
	// 확장자
	map<string, ColorEntry>::iterator exti = e_tColor.mapExt.find( tolower(t.ext()) );
	
	if (exti != e_tColor.mapExt.end() ) 
		return &(*exti).second;
	
	// 퍼미션
	int mode=0;
	
	for (int z=1; z<10; z++)
	{
		mode <<= 1;
		mode +=	t.attr[z] == '-' ? 0 : 1; 		
	}	
	for (map<int , ColorEntry>::iterator mi = e_tColor.mapMask.begin(); mi != e_tColor.mapMask.end(); ++mi)
		if (mode & (*mi).first) return &(*mi).second;				
	
	
	return &e_tColor.Default;	
}

void Panel::Sort(void)
{
	// 소팅을 시작합니다	
	if (file_list.size() > 1)
	{	
		pFileIterator i, 
			 begin = file_list.begin(),
			 end   = file_list.end();
		
		if (e_tConf.DirSort)
		{
			if (!bRoot) 
			{// ..을 찾아서 맨 앞으로(루트는 없다)
				for(i = begin; i != end; i++)
					if ((*i)->name == "..")
					{
						File *tmp = *begin;
						*begin = *i;
						*i = tmp;
						begin++;
						break;
					}
			}
			
			// 디렉토리들을 찾아서 앞으로..
			pFileIterator s=end-1;
			
			for (;begin!=end; begin++)
			{
				if (!(*begin)->bDir)
				{
					while(s != begin && !(*s)->bDir) s--;
					
					if (begin == s) break;
					
					File *tmp   = *begin;
					     *begin = *s;
					     *s     = tmp;	
				}
			}
			
			if (bRoot) sort(file_list.begin(), begin, sort_name());
			else sort(file_list.begin()+1, begin, sort_name());
		}	
	
		// 정렬	
		switch(e_tConf.sort)
		{
		case 1: // name
			sort(begin, end, sort_name());	
			break;
		case 2: // ext
			sort(begin, end, sort_ext());	
			break;
		case 3: // size
			sort(begin, end, sort_size());	
			break;	
		case 4: // time
			sort(begin, end, sort_time());	
			break;
		case 5:  // color
			sort(begin, end, sort_color());	
			break;			
		case 0:
		default:
			break;
		}
	}	
}

void Panel::Refresh(void)
{
	int tmp;
	string cur_name = file_list[cur]->name;
	
	if (isZip() == true)
	{
		string sTmpPath = GetZipCurrentPath();
		sTmpPath = sTmpPath.substr(1, sTmpPath.size()-1);
		if (Read(sTmpPath) == false)
		{
			sTmpPath = "/" + sTmpPath;
			Read(sTmpPath);
		}
	}
	else
	{
		// GetCurrentPath로 하게되면 링크된 디렉토리가 유지되지 않습니다.
		Read(".");
	}	
	
	cur = Search(cur_name);
	if (cur == -1) cur = 0;
	
}

File Panel::GetCurrentFile(void)
{
	return *file_list[cur];
}

string Panel::ViewCurrentPath(void)
{
	if (m_nZip == 0)
		return m_sTmpPath;
	else
		return m_zipPath;
}

/// \brief	현재 dir 절대 path를 구한다.
/// \return	현재 dir 절대 path
string Panel::GetCurrentPath(void)
{
	string 	s_path;
	
	// . 현재 dir name을 구한다.
	char *tmpdir  = getcwd(NULL, 0);
	s_path = tmpdir;
	free(tmpdir);
		
	// . 끝에 '/'를 추가한다.
	if (s_path != "/") s_path += '/';
	
	return s_path;
}

/// \brief	상대경로는 절대경로로 바꾼다.
/// \param	str		상대경로 path명
///	\return	절대경로명
string Panel::GetRealPath(const string &str)
{
	string s_path;
	
	s_path = str;
	
	// . zip 파일 검색이 아니면
	if (m_nZip == 0)
	{
		// .. /(root) dir이 아니면
		if (s_path[0]!='/')
		{
			// ... 현재 dir를 얻어야 한다면 현재 dir절대경로를 찾는다. 
			if (s_path == ".") s_path = path.empty() ? GetCurrentPath() : path;
			// ... 상위 dir를 얻어야 한다면
			else if (s_path == "..")
			{
				// .... /(root)dir이 아니면 상위 절대경로 dir명을 구함
				if (path != "/")
				{
					string::size_type p = path.rfind('/', path.size()-2);
					s_path = path.substr(0, p+1);
				}			
				/// .... /(root)dir이면 절대경로에 / 를 넣음
				else s_path = path;
			}			
			else // ... 그외 dir은 path에 추가하여 절대경로를 구한다.
			{
				s_path = path + s_path + '/';
			}
		}
	}
	// . zip 파일 이면
	else
	{
		// . 현재 절대경로를 구한다.
		s_path = GetCurrentPath();
	}
	
	m_sTmpPath = s_path;
	return s_path;
}

/// \brief	디렉토리에서 파일이름을 읽는다.
///
///  path 뒤에는 항상 /가 붙는 것을 원칙으로 합니다. (예 /home/iddd/ )
/// \param	dir_name	읽을 디렉토리명
///	\return	성공여부
bool Panel::Read(const string &dir_name)
{
	string prev_dir;
	string strPath;
	
	// . 읽을 디렉토리가 상위 디렉토리이고 현재 path가 / 가아니면
	if (dir_name == ".."  && path != "/")
	{
		// .. 현재 path를 prev_dir에 저장
		string::size_type p = path.rfind('/', path.size()-2);
		prev_dir = path.substr(p+1, path.size()-p-2);
	}

	// . 상대경로를 절대경로로 바꾼다.
	strPath  = GetRealPath(dir_name);
	
	//MsgBox("Error", strPath.c_str());
	// 절대경로에 맞는 Reader를 얻어온다
	m_tReader = GetReader(strPath);	
	
	if (m_tReader == NULL) return false;
	
	// . zip 파일이 아니면
	if (m_nZip == 0)
	{
		if (!m_tReader->Init(strPath)) 		return false;// .. Init 은 항상 절대경로로.
	}
	// . zip 파일이면
	else
	{
		LOG("Zip File INIT :: %s", dir_name.c_str());
		
		if (!m_tReader->Init(dir_name)) // Zip 은 dir_name으로	
		{
			//MsgBox("ERROR", "Zip file reading failure!!!");
			return false;
		}
	}

	bRoot = m_tReader->isRoot();
	
	path  = strPath;
	
	// . History에 현재 dir name 저장
	tHistory.AddEntry(path);
	
	// . panel init
	Init();
	
	while(m_tReader->Next())
	{
		File &t = pool.Get();
		
		if (!m_tReader->GetInfo(t)) continue;
		
		if (m_nZip == 0)
		{
			if (t.name == ".")	continue;
			if (bRoot && t.name == "..") continue;
			if (!e_tConf.ShowHidden && t.name[0] == '.' && t.name != ".." ) continue;
		}
		else
		{
			//if (!e_tConf.ShowHidden && t.name[0] == '.' && t.name != ".." ) continue;
		}
		
		dir_size += t.size;
		
		t.selected = false;
		t.color    = GetColor(t);
		
//		LOG("FileList Data :: %s", t.name.c_str());
		file_list.push_back(&t);
		
		if (t.bDir) ndir++;
		else nfile++;
	}
	
	Sort();	
	
	// . 바로 전 dir 에 커서를 위치시킨다.
	if (!prev_dir.empty()) // ..에 해당하는 디렉토리를 찾는다..
	{
		int tmp;
		if ((tmp = Search(prev_dir)) != -1) cur = tmp;
	}
	
	return true;
}

==> mls-0.4.tar.gz: mls-0.4/src/panel_read.cpp <==
///	\file	panel_read.cpp
///	\brief	panel_read.cpp
#include <dirent.h>
#include <sys/stat.h>
#include <time.h>
#include <string>
#include <map>
#include <algorithm>
#include <unistd.h>

#include "panel.h"
#include "conf.h"
#include "strutil.h"
#include "dialog.h"
#include "ZipReader.h"

using namespace std;
using namespace strutil;
using namespace MLS;

// read interface
/*
 * (기본) 파일 리더.
 * file:// 파일 리더
 * arc:// 
 * ftp:// 
 */
 
/// \brief	초기화 함수
void Panel::Init(void)
{
	cur      = 0;
	page     = 0;
	sel_size = 0;
	sel_num  = 0;	

	nfile    = 0;
	ndir     = 0;
	dir_size = 0;	
	
	file_list.clear();
	pool.Clear();	
}

/// \brief	파일명에 해당 되는 색상을 구함
/// \param	t		파일
/// \return	색상
ColorEntry *Panel::GetColor(File &t)
{
	// 이름
	map<string, ColorEntry>::iterator namei = e_tColor.mapName.find( tolower(t.name) );
	
	if (namei != e_tColor.mapName.end() ) 
		return &(*namei).second;
	
	// 확장자
	map<string, ColorEntry>::iterator exti = e_tColor.mapExt.find( tolower(t.ext()) );
	
	if (exti != e_tColor.mapExt.end() ) 
		return &(*exti).second;
	
	// 퍼미션
	int mode=0;
	
	for (int z=1; z<10; z++)
	{
		mode <<= 1;
		mode +=	t.attr[z] == '-' ? 0 : 1; 		
	}	
	for (map<int , ColorEntry>::iterator mi = e_tColor.mapMask.begin(); mi != e_tColor.mapMask.end(); ++mi)
		if (mode & (*mi).first) return &(*mi).second;				
	
	
	return &e_tColor.Default;	
}

void Panel::Sort(void)
{
	// 소팅을 시작합니다	
	if (file_list.size() > 1)
	{	
		pFileIterator i, 
			 begin = file_list.begin(),
			 end   = file_list.end();
		
		if (e_tConf.DirSort)
		{
			if (!bRoot) 
			{// ..을 찾아서 맨 앞으로(루트는 없다)
				for(i = begin; i != end; i++)
					if ((*i)->name == "..")
					{
						File *tmp = *begin;
						*begin = *i;
						*i = tmp;
						begin++;
						break;
					}
			}
			
			// 디렉토리들을 찾아서 앞으로..
			pFileIterator s=end-1;
			
			for (;begin!=end; begin++)
			{
				if (!(*begin)->bDir)
				{
					while(s != begin && !(*s)->bDir) s--;
					
					if (begin == s) break;
					
					File *tmp   = *begin;
					     *begin = *s;
					     *s     = tmp;	
				}
			}
			
			if (bRoot) sort(file_list.begin(), begin, sort_name());
			else sort(file_list.begin()+1, begin, sort_name());
		}	
	
		// 정렬	
		switch(e_tConf.sort)
		{
		case 1: // name
			sort(begin, end, sort_name());	
			break;
		case 2: // ext
			sort(begin, end, sort_ext());	
			break;
		case 3: // size
			sort(begin, end, sort_size());	
			break;	
		case 4: // time
			sort(begin, end, sort_time());	
			break;
		case 5:  // color
			sort(begin, end, sort_color());	
			break;			
		case 0:
		default:
			break;
		}
	}	
}

void Panel::Refresh(void)
{
	int tmp;
	string cur_name = file_list[cur]->name;
	
	if (isZip() == true)
	{
		string sTmpPath = GetZipCurrentPath();
		sTmpPath = sTmpPath.substr(1, sTmpPath.size()-1);
		if (Read(sTmpPath) == false)
		{
			sTmpPath = "/" + sTmpPath;
			Read(sTmpPath);
		}
	}
	else
	{
		// GetCurrentPath로 하게되면 링크된 디렉토리가 유지되지 않습니다.
		Read(".");
	}	
	
	cur = Search(cur_name);
	if (cur == -1) cur = 0;
	
}

File Panel::GetCurrentFile(void)
{
	return *file_list[cur];
}

string Panel::ViewCurrentPath(void)
{
	if (m_nZip == 0)
		return m_sTmpPath;
	else
		return m_zipPath;
}

/// \brief	현재 dir 절대 path를 구한다.
/// \return	현재 dir 절대 path
string Panel::GetCurrentPath(void)
{
	string 	s_path;
	
	// . 현재 dir name을 구한다.
	char *tmpdir  = getcwd(NULL, 0);
	s_path = tmpdir;
	free(tmpdir);
		
	// . 끝에 '/'를 추가한다.
	if (s_path != "/") s_path += '/';
	
	return s_path;
}

/// \brief	상대경로는 절대경로로 바꾼다.
/// \param	str		상대경로 path명
///	\return	절대경로명
string Panel::GetRealPath(const string &str)
{
	string s_path;
	
	s_path = str;
	
	// . zip 파일 검색이 아니면
	if (m_nZip == 0)
	{
		// ~ 인지 먼저 판단.
		if (s_path[0]=='~')
		{			
			if (s_path == "~" || s_path[1] == '/')
			{
				// . home 디렉토리 지정
				struct passwd *pw = getpwuid(getuid()); // getuid이냐 geteuid냐...;
				string home  = pw->pw_dir;
				home += '/';
				string s_tmp = s_path.substr(1);
				string s_tmp2 = "";
				if (s_tmp.find('/') != -1)
					s_tmp2 = s_tmp.substr(s_tmp.find('/')+1);
				s_path = home + s_tmp2;
			}
			else
			{	
				string s_tmp = s_path.substr(1);
				string s_tmp2 = "", s_tmp3 = "";
				if (s_tmp.find('/') != -1)
					s_tmp2 = s_tmp.substr(0, s_tmp.find('/'));
				else
					s_tmp2 = s_tmp;
				if (s_tmp.find('/') != -1)
					s_tmp3 = s_tmp.substr(s_tmp.find('/')+1);
				struct passwd *pw = getpwnam(s_tmp2.c_str());
				if (pw == NULL) return "";
				string home = pw->pw_dir;
				home += '/';
				s_path = home + s_tmp3;
			}
		}
		else if (s_path[0]!='/') // .. /(root) dir이 아니면
		{
			// ... 현재 dir를 얻어야 한다면 현재 dir절대경로를 찾는다. 
			if (s_path == ".") s_path = path.empty() ? GetCurrentPath() : path;
			// ... 상위 dir를 얻어야 한다면
			else if (s_path == "..")
			{
				// .... /(root)dir이 아니면 상위 절대경로 dir명을 구함
				if (path != "/")
				{
					string::size_type p = path.rfind('/', path.size()-2);
					s_path = path.substr(0, p+1);
				}			
				/// .... /(root)dir이면 절대경로에 / 를 넣음
				else s_path = path;
			}			
			else // ... 그외 dir은 path에 추가하여 절대경로를 구한다.
			{
				s_path = path + s_path;
			}
		}
	}
	// . zip 파일 이면
	else
	{
		// . 현재 절대경로를 구한다.
		s_path = GetCurrentPath();
	}
	if (s_path.substr(s_path.size()-1, 1) != "/") s_path += '/';
	m_sTmpPath = s_path;
	return s_path;
}

/// \brief	디렉토리에서 파일이름을 읽는다.
///
///  path 뒤에는 항상 /가 붙는 것을 원칙으로 합니다. (예 /home/iddd/ )
/// \param	dir_name	읽을 디렉토리명
///	\return	성공여부
bool Panel::Read(const string &dir_name)
{
	string prev_dir;
	string strPath;
	
	// . 읽을 디렉토리가 상위 디렉토리이고 현재 path가 / 가아니면
	if (dir_name == ".."  && path != "/")
	{
		// .. 현재 path를 prev_dir에 저장
		string::size_type p = path.rfind('/', path.size()-2);
		prev_dir = path.substr(p+1, path.size()-p-2);
	}

	// . 상대경로를 절대경로로 바꾼다.
	strPath  = GetRealPath(dir_name);
	
	//MsgBox("Error", strPath.c_str());
	// 절대경로에 맞는 Reader를 얻어온다
	m_tReader = GetReader(strPath);
	
	if (m_tReader == NULL) return false;
	
	// . zip 파일이 아니면
	if (m_nZip == 0)
	{
		if (!m_tReader->Init(strPath)) 		return false;// .. Init 은 항상 절대경로로.
	}
	// . zip 파일이면
	else
	{
		LOG("Zip File INIT :: %s", dir_name.c_str());
		
		if (!m_tReader->Init(dir_name)) // Zip 은 dir_name으로	
		{
			//MsgBox("ERROR", "Zip file reading failure!!!");
			return false;
		}
	}

	bRoot = m_tReader->isRoot();
	
	path  = strPath;
	
	// . History에 현재 dir name 저장
	tHistory.AddEntry(path);
	
	// . panel init
	Init();
	
	while(m_tReader->Next())
	{
		File &t = pool.Get();
		
		if (!m_tReader->GetInfo(t)) continue;
		
		if (m_nZip == 0)
		{
			if (t.name == ".")	continue;
			if (bRoot && t.name == "..") continue;
			if (!e_tConf.ShowHidden && t.name[0] == '.' && t.name != ".." ) continue;
		}
		else
		{
			//if (!e_tConf.ShowHidden && t.name[0] == '.' && t.name != ".." ) continue;
		}
		
		dir_size += t.size;
		
		t.selected = false;
		t.color    = GetColor(t);
		
//		LOG("FileList Data :: %s", t.name.c_str());
		file_list.push_back(&t);
		
		if (t.bDir) ndir++;
		else nfile++;
	}
	
	Sort();	
	
	// . 바로 전 dir 에 커서를 위치시킨다.
	if (!prev_dir.empty()) // ..에 해당하는 디렉토리를 찾는다..
	{
		int tmp;
		if ((tmp = Search(prev_dir)) != -1) cur = tmp;
	}
	
	return true;
}
